// GLSL vertex shader for terrain texturing by Peter Alexay, 2006
// Texture coordinates are the same for all texture units.
// Input:
// - in gl_Color.rgb are the coverage factors
// - in gl_MultitexCoord0,1,2,3 are the u/v 's for each texture unit
// - gl_TextureMatrix[0..3] is the texture space matrix for each texture unit
// - gl_Normal is the current vertex normal
// - gl_LightSource[0] is the directional light source (Sun). Position and ambient+diffuse color should be set.
// - gl_FrontMaterial is the terrain's material color (ambient + diffuse)
// - bool fogCoordsPresent is true only if per-vertex fog-coords are set (volumetric mode)
// Output:
// - in 'coverage' are the interpolated coverage factors for the fragments
// - in gl_TexCoord[0..3] are the interpolated texture coordinates
// - the shaded color that is computed using diffuse lighting is available as fragment color

uniform float fogEnabled; // true if we want fog on the terrain
uniform float fogCoordsPresent; // if fog type is 'volumetric', fog coords per vertex are presented
//uniform bool fogEnabled; // true if we want fog on the terrain
//uniform bool fogCoordsPresent; // if fog type is 'volumetric', fog coords per vertex are presented
uniform float fogMult; // precomputed value for fog exponent : -fog_density*(1/log(2))
varying float fogValue; // the blend factor for the fragment shader
varying vec4 coverage;

void main()
{
  vec3 normal;
  vec4 ecPos; // eye-space (clip space) vertex position
  float NdotL;
  
  coverage=gl_Color;
  
  normal=normalize(gl_NormalMatrix*gl_Normal);
  //lightDir=normalize(vec3(gl_LightSource[0].position));
  NdotL=max(dot(normal,normalize(vec3(gl_LightSource[0].position))),0.0);
  //globalAmbient=gl_LightModel.ambient*gl_FrontMaterial.ambient;
  gl_FrontColor=NdotL*gl_FrontMaterial.diffuse*gl_LightSource[0].diffuse+
                gl_FrontMaterial.ambient*gl_LightSource[0].ambient+
                gl_LightModel.ambient*gl_FrontMaterial.ambient;
  ecPos=gl_ModelViewMatrix*gl_Vertex;
  
/* WITH IFs
  if (fogEnabled)
  {
    if (fogCoordsPresent)
      gl_FogFragCoord=gl_FogCoord;
    else
      gl_FogFragCoord=abs(ecPos.z);
  
    fogValue=clamp(exp2(gl_FogFragCoord*fogMult),0.0,1.0); // GL_EXP fog
  }
  else
  {
    fogValue=1.0; // disable fog in the fragment shader
  }
*/
  
  gl_FogFragCoord=gl_FogCoord*fogCoordsPresent + abs(ecPos.z)*(1.0-fogCoordsPresent);
  fogValue=(1.0-fogEnabled)+(clamp(exp2(gl_FogFragCoord*fogMult),0.0,1.0)*fogEnabled);
  
  gl_TexCoord[0]=gl_TextureMatrix[0]*gl_MultiTexCoord0;
  gl_TexCoord[1]=gl_TextureMatrix[1]*gl_MultiTexCoord1;
  gl_TexCoord[2]=gl_TextureMatrix[2]*gl_MultiTexCoord2;
  gl_TexCoord[3]=gl_TextureMatrix[3]*gl_MultiTexCoord3;
  gl_Position=ftransform();
  gl_ClipVertex=ecPos;
}
